# Written by retoor@molodetz.nl

# This code defines a CallQueue class that queues function calls and executes them sequentially, supporting both synchronous and asynchronous functions.

# Imports from standard modules: functools, deque, asyncio

# MIT License
#
# (C) 2023 The Author
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
#
# Copies or substantial portions of the Software are reproduced and must include the above copyright notice and other similar permission notice in this particular software, including in any accompanying documentation.


import asyncio
from collections import deque
from functools import wraps


class CallQueue:
    def __init__(self):
        self.call_queue = deque()

    def __call__(self, func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            self.call_queue.append((func, args, kwargs))

        return wrapper

    async def execute_next(self):
        if self.call_queue:
            func, args, kwargs = self.call_queue.popleft()
            if asyncio.iscoroutinefunction(func):
                return await func(*args, **kwargs)
            else:
                return func(*args, **kwargs)
        else:
            raise IndexError("The call queue is empty.")

    async def execute_all(self):
        results = []
        while self.call_queue:
            results.append(await self.execute_next())
        return results
